function zM = AAFTsur(xV,nsur)
% zM = AAFTsur(xV,nsur)
% AAFTsur: Amplitude Adjusted Fourier Transform Surrogates
% This function generates 'nsur' AAFT-surrogate time series and stores them 
% in the matrix 'zM' (columnwise). The AAFT surrogates are supposed to have 
% the same amplitude distribution (marginal cdf) and autocorrelation as the 
% given time series 'xV'. 
% The algorithm is according to 
% Theiler, J. et al (1992), "Testing for Nonlinearity in Time Series: 
% the Method of Surrogate Data", Physica D, Vol 58, 77-94.
% Refer also to the Appendix in 
% Theiler, J. et al (1992), "Using Surrogate Data to Detect Nonlinearity 
% in Time Series", in "Nonlinear Modeling and Forecasting", edited by
% Casdagli, M. and Eubank, S., Addison-Wesley, Reading, MA, 163-188.
% INPUT
% - xV  : the given time series
% - nsur: the number of surrogate time series (default is 1)
% OUTPUT
% - zM  : the n x nsur matrix of 'nsur' AAFT surrogate time series

if nargin==1
    nsur=1;
end
n = length(xV);
zM = NaN*ones(n,nsur);
[oxV,T] = sort(xV);  
[T,ixV] = sort(T); 
for isur=1:nsur
    % Rank order a white noise time series 'wV' to match the ranks of 'xV' 
    wV = randn(n,1) * std(xV);
    [owV,T]= sort(wV); 
    yV = owV(ixV);
    % Fourier transform, phase randomization, inverse Fourier transform
    if rem(n,2) == 0
        n2 = n/2;
    else
        n2 = (n-1)/2;
    end
    tmpV = fft(yV,2*n2);
    magnV = abs(tmpV);
    fiV = angle(tmpV);
    rfiV = rand(n2-1,1)*2*pi;
    nfiV = [0; rfiV; fiV(n2+1); -flipud(rfiV)];
    tmpV = [magnV(1:n2+1)' flipud(magnV(2:n2))']';
    tmpV = tmpV .* exp(nfiV .* i); 
    yftV=real(ifft(tmpV,n));  % Transform back to time domain
    % Rank order the 'xV' to match the ranks of the phase randomized time series 
    [T,T2] = sort(yftV); 
    [T,iyftV] = sort(T2);
    zM(:,isur) = oxV(iyftV);  % the AAFT surrogate of xV
end
