function [bicV,cumbicV] = Bicorrelation(xV,tauV,flag)
% [bicV,cumbicV] = Bicorrelation(xV,tauV,flag)
% BICORRELATION computes the bicorrelation on the time series 'xV' for 
% given delays in 'tauV'. In addition, it computes the cumulative 
% bicorrelation for each given lag.
% Bicorrelation is the extension of the autocorrelation to the third order 
% moments, where the two delays are chosen here to be the one twice the
% other, i.e. x(t)*x(t-tau)*x(t-2*tau). 
% According to a given flag, it can also compute the cumulative
% bicorrelation for each given lag.
% INPUT
% - xV      : a vector for the time series
% - tauV    : a vector of the delays to be evaluated for
% - flag    : if 0-> compute only mutual information,
%           : if 1-> compute the mutual information, the first minimum of
%             if 2-> compute only the cumulative bicorrelation.
% OUTPUT
% - bicV    : the vector of the bicorrelations for the given delays
% - cumbicV : the vector of the cumulative bicorrelations for the
%             given delays 

tauV(tauV==0)=[];
tauV = sort(tauV);
ntau = length(tauV);
taumax = tauV(end);
n = length(xV);

if n<tauV(end)+1
    return;
end
yV = (xV - mean(xV)) / std(xV);
switch flag
    case 0
        % Compute only the bicorrelation for the given lags
        bicV = NaN*ones(ntau,1);
        for itau=1:ntau
            tau = tauV(itau);
            bicV(itau)= sum(yV(1+2*tau:n).*yV(1+tau:n-tau).*yV(1:n-2*tau))/(n-2*tau);
        end
        cumbicV = [];
    case 1
        % Compute the bicorrelation for all lags up to the largest given
        % lag and the cumulative bicorrelation for the given lags.
        biV = NaN*ones(taumax,1);
        for tau=1:taumax
            biV(tau)= sum(yV(1+2*tau:n).*yV(1+tau:n-tau).*yV(1:n-2*tau))/(n-2*tau);
        end
        bicV = biV(tauV); % store only the delays given in 'tauV' 
        % Compute the cumulative bicorrelation for the given delays
        cumbicV = NaN*ones(ntau,1);
        for i=1:ntau
            cumbicV(i) = sum(abs(biV(1:tauV(i))));
        end
    case 2
        % Compute the cumulative bicorrelation for the given lags.
        biV = NaN*ones(taumax,1);
        for tau=1:taumax
            biV(tau)= sum(yV(1+2*tau:n).*yV(1+tau:n-tau).*yV(1:n-2*tau))/(n-2*tau);
        end
        bicV = []; % 
        % Compute the cumulative bicorrelation for the given delays
        cumbicV = NaN*ones(ntau,1);
        for i=1:ntau
            cumbicV(i) = sum(abs(biV(1:tauV(i))));
        end
end