function xM = DivideSeg(xV,n,s,first)
% xM = DivideSeg(xV,n,s,first)
% DIVIDESEG takes a time series 'xV' and splits in segments of length 'n'. 
% The segments either overlap if 's' is smaller than 'n' or not (otherwise).
% If the length of 'xV' is not multiple of 'n', then the remainder segment 
% is omitted either from the beginning of 'xV' when 'first' is 1, or from
% the end (otherwise).
% INPUTS:
% - xV      : vector of a scalar time series
% - n       : length of each of the segments splitting 'xV'.
% - s       : the sliding window if 0<s<n, otherwise use consecutive
%             segments. Defaults is n. 
% - first   : if 1, ignore the remainder segment (only if length(xV) is not
%             multiple of n) from the beginning of 'xV', otherwise from the
%             end of 'xV'. Default is 1.
% OUTPUTS:
% - xM      : a matrix of size n x floor(length(xV)/n) of the segments.

% Give default values to input variables if not set
if nargin == 3
    first = 1;
elseif nargin == 2
    first = 1;
    s = n;
end
if isempty(first), first=1; end
if isempty(s), s=n; end
if s<=0 | s>n, s=n; end

nbig = length(xV);
if n>nbig
    return;
end

% Make splitting if no-overlapping is given and then when 0<s<n
if s==n
    nseg = floor(nbig / n);
    nrest=nbig - nseg*n;
    if nrest>0
        switch first
            case 1
                xV = xV(1+nrest:nbig);
            otherwise
                xV = xV(1:nbig-nrest);
        end
    end
    xM = reshape(xV,n,nseg);
else
    nseg = floor((nbig-n)/s)+1;
    nrest = nbig - (n+(nseg-1)*s);
    if nrest>0
        switch first
            case 1
                xV = xV(1+nrest:nbig);
            otherwise
                xV = xV(1:nbig-nrest);
        end
    end
    xM = NaN*ones(n,nseg);
    for iseg=1:nseg
        xM(:,iseg)=xV(1+(iseg-1)*s:n+(iseg-1)*s);
    end
end
    