function fnnM = FalseNearestNeighbors(xV,tauV,mV,escape,theiler)
% fnnM = FalseNearestNeighbors(xV,tauV,mV,escape,theiler)
% FALSENEARESTNEIGHBORS computes the percentage of false nearest neighbors
% for a range of delays in 'tauV' and embedding dimensions in 'mV'.
% INPUT 
%  xV       : Vector of the scalar time series
%  tauV     : A vector of the delay times.
%  mV       : A vector of the embedding dimension.
%  escape   : A factor of escaping from the neighborhood. Default=10.
%  theiler  : the Theiler window to exclude time correlated points in the
%             search for neighboring points. Default=0.
% OUTPUT: 
%  fnnM     : A matrix of size 'ntau' x 'nm', where 'ntau' is the number of
%             given delays and 'nm' is the number of given embedding
%             dimensions, containing the percentage of false nearest
%             neighbors.

fthres = 0.1; % A factor of the data SD to be used as the maximum radius 
              % for searching for valid nearest neighbor.
propthres = fthres; % Limit for the proportion of valid points, i.e. points 
                    % for which a nearest neighbor was found. If the proporion 
                    % of valid points is beyond this limit, do not compute
                    % FNN.
              
n = length(xV);
if isempty(escape), escape=10; end
if isempty(theiler), theiler=0; end
% Rescale to [0,1] and add infinitesimal noise to have distinct samples
tic
xmin = min(xV);
xmax = max(xV);
xV = (xV - xmin) / (xmax-xmin);
t=toc;
xV = AddNoise(xV,10^(-10));
rthres = fthres*std(xV); % The maximum distance to look for nearest neighbor
ntau = length(tauV);
nm = length(mV);
fnnM = NaN*ones(ntau,nm);
for itau = 1:ntau
    tau = tauV(itau);
    for im=1:nm
        m = mV(im);
        nvec = n-m*tau; % to be able to add the component x(nvec+tau) for m+1 
        if nvec-theiler < 2
            break;
        end
        xM = NaN*ones(nvec,m);
        for i=1:m
            xM(:,m-i+1) = xV(1+(i-1)*tau:nvec+(i-1)*tau);
        end
        % k-d-tree data structure of the training set for the given m
        [tmp,tmp,TreeRoot]=kdtreeidx(xM,[]); 
        % For each target point, find the nearest neighbor, and check whether 
        % the distance increase over the escape distance by adding the next
        % component for m+1.
        idxV = NaN*ones(nvec,1);
        distV = NaN*ones(nvec,1);
        for i=1:nvec
            tarV = xM(i,:);
            [neiM,neidisV,neiindV]=kdrangequery(TreeRoot,tarV,rthres*sqrt(m));
            [oneidisV,oneiindV]=sort(neidisV);
            neiindV = neiindV(oneiindV);
            neidisV = neidisV(oneiindV);
            iV = find(abs(neiindV(1)-neiindV(2:end))>theiler);
            if ~isempty(iV)
                idxV(i) = neiindV(iV(1)+1);
                distV(i) = neidisV(iV(1)+1);
            end
        end % for i
        iV = find(~isnan(idxV));
        nproper = length(iV);
        % Compute fnn only if there is a sufficient number of target points 
        % having nearest neighbor (in R^m) withing the threshold distance
        if nproper>propthres*nvec
            nnfactorV = 1+(xV(iV+m*tau)-xV(idxV(iV)+m*tau)).^2./distV(iV).^2;
            fnnM(itau,im) = length(find(nnfactorV > escape^2))/nproper;
        end
        kdtreeidx([],[],TreeRoot); % Free the pointer to k-d-tree
    end
end
