function [mutM,cummutM,minmuttauV] =  MutualInformationHisDis(xV,tauV,bV,flag)
% [mutM,cummutM,minmuttauV] =  MutualInformationHisDis(xV,tauV,bV,flag)
% MUTUALINFORMATIONHISDIS computes the mutual information on the time
% series 'xV' for given delays in 'tauV'. The estimation of mutual
% information is based on 'b' partitions of equal distance at each dimension. 
% A number of different 'b' can be given in the input vector 'bV'.
% According to a given flag, it can also compute the cumulative mutual 
% information for each given lag, as well as the time of the first minimum 
% of the mutual information.
% INPUT
% - xV      : a vector for the time series
% - tauV    : a vector of the delays to be evaluated for
% - bV      : a vector of the number of partitions of the histogram-based
%             estimate. 
% - flag    : if 0-> compute only mutual information,
%           : if 1-> compute the mutual information, the first minimum of
%             mutual information and the cumulative mutual information. 
%             if 2-> compute (also) the cumulative mutual information
%             if 3-> compute (also) the first minimum of mutual information
% OUTPUT
% - mutM    : the vector of the mutual information values s for the given
%             delays. 
% - cummutM : the vector of the cumulative mutual information values for
%             the given delays 
% - minmuttauV : the time of the first minimum of the mutual information.

nsam = 1;
n = length(xV);
if nargin==3
    flag = 1;
elseif nargin==2 
    flag = 1;
    bV = round(sqrt(n/5));
end
if isempty(bV)
    bV = round(sqrt(n/5));
end
bV(bV==0)=round(sqrt(n/5));
tauV = sort(tauV);
ntau = length(tauV);
taumax = tauV(end);
% Normalise the data
xmin = min(xV);
[xmax,imax] = max(xV);
xV(imax) = xmax + (xmax-xmin)*10^(-10); % To avoid multiple exact maxima
yV = (xV-xmin)/(xmax-xmin);
nb = length(bV);

switch flag
    case 0
        % Compute only the mutual information for the given lags
        mutM = NaN*ones(ntau,nb);
        for ib=1:nb
            b = bV(ib);
            if n<2*b
                break;
            end
            mutM(:,ib)=mutinfHisDis(yV,tauV,b,imax);
        end % for ib
        cummutM=[];
        minmuttauV=[];
    case 1
        % Compute the mutual information for all lags up to the
        % largest given lag, then compute the lag of the first minimum of
        % mutual information and the cumulative mutual information for the
        % given lags.
        mutM = NaN*ones(ntau,nb);
        cummutM = NaN*ones(ntau,nb);
        minmuttauV = NaN*ones(nb,1);
        miM = NaN*ones(taumax+1,nb);
        for ib=1:nb
            b = bV(ib);
            if n<2*b
                break;
            end
            miM(:,ib)=mutinfHisDis(yV,[0:taumax]',b,imax);
            mutM(:,ib) = miM(tauV+1,ib);
            minmuttauV(ib) = findminMutInf(miM(:,ib),nsam);
            % Compute the cumulative mutual information for the given delays
            for i=1:ntau
                cummutM(i,ib) = sum(miM(1:tauV(i)+1,ib));
            end
        end % for ib
    case 2
        % Compute the mutual information for all lags up to the largest 
        % given lag and then sum up to get the cumulative mutual information 
        % for the given lags.
        cummutM = NaN*ones(ntau,nb);
        miM = NaN*ones(taumax+1,nb);
        for ib=1:nb
            b = bV(ib);
            if n<2*b
                break;
            end
            miM(:,ib)=mutinfHisDis(yV,[0:taumax]',b,imax);
            % Compute the cumulative mutual information for the given delays
            for i=1:ntau
                cummutM(i,ib) = sum(miM(1:tauV(i)+1,ib));
            end
        end % for ib
        mutM = [];
        minmuttauV=[];
    case 3
        % Compute the mutual information for all lags up to the largest 
        % given lag and then compute the lag of the first minimum of the
        % mutual information.
        minmuttauV = NaN*ones(nb,1);
        miM = NaN*ones(taumax+1,nb);
        for ib=1:nb
            b = bV(ib);
            if n<2*b
                break;
            end
            miM(:,ib)=mutinfHisDis(yV,[0:taumax]',b,imax);
            minmuttauV(ib) = findminMutInf(miM(:,ib),nsam);
        end % for ib
        mutM = [];
        cummutM=[];
end