function [sautV,cumsautV,decsaut,zerosaut] = Spearmanautocorrelation(xV,tauV,flag)
% [sautV,cumsautV,decsaut,zerosaut] = Spearmanautocorrelation(xV,tauV,flag)
% SPEARMANAUTOCORRELATION computed the Spearman autocorrelation on the time
% series 'xV' for given delays in 'tauV'.
% According to a given flag, it can also compute the cumulative Spearman
% autocorrelation for each given lag, the decorrelation time (the delay
% for which the Spearman autocorrelation is 1/e) and the time the
% Spearman autocorrelation falls to zero.
% INPUT
% - xV       : a vector for the time series
% - tauV     : a vector of the delays to be evaluated for
% - flag    : if 0-> compute only Spearman autocorrelation,
%           : if 1-> compute the Spearman autocorrelation, the cumulative
%             Spearman autocorrelation, the decorrelation time and the time
%             of zero Spearman autocorrelation.
%             if 2-> compute (also) the cumulative Spearman autocorrelation.
%             if 3-> compute (also) the decorrelation time.
%             if 4-> compute (also) the time of zero Spearman autocorrelation.
% OUTPUT
% - sautV    : the vector of the Spearman autocorrelations for the given delays
% - cumsautV : the vector of the cumulative Spearman autocorrelations for the
%              given delays
% - decsaut  : the decorrelation time from Spearman autocorrelation.
% - zerosaut : the time the Spearman autocorrelation falls to zero.

if nargin==2
    flag = 1;
end
tauV = sort(tauV);
ntau = length(tauV);
taumax = tauV(end);
n = length(xV);
if n<taumax+1
    return;
end
sautV=NaN;
cumsautV=NaN;
decsaut=NaN;
zerosaut=NaN;

switch flag
    case 0
        % Compute only the Spearman autocorrelation for the given lags
        sautV=spearmancor(xV,tauV);
    case 1
        % Compute the Spearman autocorrelation for the given lags, the
        % cumulative Spearman autocorrelation, and find, if possible,
        % the decorrelation time and the zero Spearman autocorrelation time
        sacV=spearmancor(xV,[1:taumax]');
        sautV = sacV(tauV); % store only the delays given in 'tauV'
        % Compute the cumulative Spearman autocorrelation for the given delays
        cumsautV = NaN*ones(ntau,1);
        for i=1:ntau
            cumsautV(i) = sum(abs(sacV(1:tauV(i))));
        end
        % Find, if possible, the decorrelation time
        dectauV = find(sacV <= 1/exp(1));
        if ~isempty(dectauV)
            decsaut = dectauV(1);
        end
        % Find, if possible, the zero Spearman autocorrelation time
        zerotauV = find(sacV <= 0);
        if ~isempty(zerotauV)
            zerosaut = zerotauV(1);
        end
    case 2
        % Compute the cumulative Spearman autocorrelation for the given lags
        sacV=spearmancor(xV,[1:taumax]');
        % Compute the cumulative Spearman autocorrelation for the given delays
        cumsautV = NaN*ones(ntau,1);
        for i=1:ntau
            cumsautV(i) = sum(abs(sacV(1:tauV(i))));
        end
    case 3
        % Find, if possible, the decorrelation time
        sacV=spearmancor(xV,[1:taumax]');
        dectauV = find(sacV <= 1/exp(1));
        if ~isempty(dectauV)
            decsaut = dectauV(1);
        end
    case 4
        % Find, if possible, the zero Spearman autocorrelation time
        sacV=spearmancor(xV,[1:taumax]');
        zerotauV = find(sacV <= 0);
        if ~isempty(zerotauV)
            zerosaut = zerotauV(1);
        end
end

